import sys
import spotipy
import spotipy.util as util

import random



# - Steps:
#     - 1. Authenticating Spotipy (✅)
#     - 2. Creating a list of your favorite artists (get max 50) ✅)
#         - API: Get a User’s Top Artists and Tracks
#         - Scope: user-top-read
#     - 3. For each of the artists, get all tracks for each artist. ✅
#         - API: Get an Artist’s Top Tracks
#     - 4. From top tracks, select tracks that are within a certain valence range ✅
#         - API: Get Audio Features for Several Tracks
#     - 5. From these tracks, create a playlist for user ✅
#         - API: Create a Playlist
#         - API: Add Tracks to a Playlist
#         - Scope: playlist-modify-public

client_id = "a00b63da8b584737931130f683d674d3"
client_secret = "47d3e9420c2f4160a6e0afbd250473b0"
redirect_uri = "http://127.0.0.1:5000"

scope = 'user-library-read user-top-read playlist-modify-public user-follow-read'

if len(sys.argv) > 1:
    username = sys.argv[1]
    mood = float(sys.argv[2])
else:
    print("Usage: %s username" % (sys.argv[0],))
    sys.exit()

token = util.prompt_for_user_token(username, scope, client_id, client_secret, redirect_uri)

if token:
	
	#Step 1. Authenticating Spotipy

	def authenticate_spotify():
		print('...connecting to Spotify')
		sp = spotipy.Spotify(auth=token)
		return sp

    #Step 2. Creating a list of your favorite artists

	def aggregate_top_artists(sp):
		print('...getting your top artists')
		top_artists_name = []
		top_artists_uri = []

		ranges = ['short_term', 'medium_term', 'long_term']
		for r in ranges:
			top_artists_all_data = sp.current_user_top_artists(limit=50, time_range= r)
			top_artists_data = top_artists_all_data['items']
			for artist_data in top_artists_data:
				if artist_data["name"] not in top_artists_name:		
					top_artists_name.append(artist_data['name'])
					top_artists_uri.append(artist_data['uri'])

		followed_artists_all_data = sp.current_user_followed_artists(limit=50)
		followed_artists_data = (followed_artists_all_data['artists'])
		for artist_data in followed_artists_data["items"]:
			if artist_data["name"] not in top_artists_name:
				top_artists_name.append(artist_data['name'])
				top_artists_uri.append(artist_data['uri'])
		return top_artists_uri


    #Step 3. For each of the artists, get a set of tracks for each artist
    
	def aggregate_top_tracks(sp, top_artists_uri):
		print("...getting top tracks")
		top_tracks_uri = []
		for artist in top_artists_uri:
			top_tracks_all_data = sp.artist_top_tracks(artist)
			top_tracks_data = top_tracks_all_data['tracks']
			for track_data in top_tracks_data:
				top_tracks_uri.append(track_data['uri'])
		return top_tracks_uri

	# Step 4. From top tracks, select tracks that are within a certain mood range

	def select_tracks(sp, top_tracks_uri):
		
		print("...selecting tracks")
		selected_tracks_uri = []

		def group(seq, size):
			return (seq[pos:pos + size] for pos in range(0, len(seq), size))

		random.shuffle(top_tracks_uri)
		for tracks in list(group(top_tracks_uri, 50)):
			tracks_all_data = sp.audio_features(tracks)
			for track_data in tracks_all_data:
				try:
					if mood < 0.10:
						if (0 <= track_data["valence"] <= (mood + 0.15)
						and track_data["danceability"] <= (mood*8)
						and track_data["energy"] <= (mood*10)):
							selected_tracks_uri.append(track_data["uri"])					
					elif 0.10 <= mood < 0.25:						
						if ((mood - 0.075) <= track_data["valence"] <= (mood + 0.075)
						and track_data["danceability"] <= (mood*4)
						and track_data["energy"] <= (mood*5)):
							selected_tracks_uri.append(track_data["uri"])
					elif 0.25 <= mood < 0.50:						
						if ((mood - 0.05) <= track_data["valence"] <= (mood + 0.05)
						and track_data["danceability"] <= (mood*1.75)
						and track_data["energy"] <= (mood*1.75)):
							selected_tracks_uri.append(track_data["uri"])
					elif 0.50 <= mood < 0.75:						
						if ((mood - 0.075) <= track_data["valence"] <= (mood + 0.075)
						and track_data["danceability"] >= (mood/2.5)
						and track_data["energy"] >= (mood/2)):
							selected_tracks_uri.append(track_data["uri"])
					elif 0.75 <= mood < 0.90:						
						if ((mood - 0.075) <= track_data["valence"] <= (mood + 0.075)
						and track_data["danceability"] >= (mood/2)
						and track_data["energy"] >= (mood/1.75)):
							selected_tracks_uri.append(track_data["uri"])
					elif mood >= 0.90:
						if ((mood - 0.15) <= track_data["valence"] <= 1
						and track_data["danceability"] >= (mood/1.75)
						and track_data["energy"] >= (mood/1.5)):
							selected_tracks_uri.append(track_data["uri"])
				except TypeError as te:
					continue

		return selected_tracks_uri			

	# Step 5. From these tracks, create a playlist for user

	def create_playlist(sp, selected_tracks_uri):

		print("...creating playlist")
		user_all_data = sp.current_user()
		user_id = user_all_data["id"]

		playlist_all_data = sp.user_playlist_create(user_id, "Moody " + str(mood))
		playlist_id = playlist_all_data["id"]

		random.shuffle(selected_tracks_uri)
		sp.user_playlist_add_tracks(user_id, playlist_id, selected_tracks_uri[0:30])


	spotify_auth = authenticate_spotify()
	top_artists = aggregate_top_artists(spotify_auth)
	top_tracks = aggregate_top_tracks(spotify_auth, top_artists)
	selected_tracks = select_tracks(spotify_auth, top_tracks)
	create_playlist(spotify_auth, selected_tracks)



else:
    print("Can't get token for", username)

